@@ -3,10 +3,11 @@ package Module::Setup;
 use strict;
 use warnings;
 use 5.008001;
-our $VERSION = '0.03_03';
+our $VERSION = '0.06_01';
 
 use Carp ();
 use Class::Trigger;
+use Cwd ();
 use ExtUtils::MakeMaker qw(prompt);
 use File::HomeDir;
 use File::Path;
@@ -15,8 +16,11 @@ use Getopt::Long;
 use Path::Class;
 use Pod::Usage;
 
+use Module::Setup::Devel;
 use Module::Setup::Distribute;
 use Module::Setup::Path;
+use Module::Setup::Path::Flavor;
+use Module::Setup::Path::Template;
 
 our $HAS_TERM;
 
@@ -25,41 +29,62 @@ sub config     { shift->{config} }
 sub options    { shift->{options} }
 sub base_dir   { shift->{base_dir} }
 sub distribute { shift->{distribute} }
+sub plugins_stash { shift->{plugins_stash} }
 
 sub new {
     my($class, %args) = @_;
 
     $args{options} ||= +{};
     $args{argv}    ||= +[];
+    $args{_current_dir} = Cwd::getcwd;
 
-    bless { %args }, $class;
+    my $self = bless { %args }, $class;
+    $self->{_current_dir}  = Cwd::getcwd;
+    $self->{plugins_stash} = +{};
+
+    $self;
+}
+
+sub DESTROY {
+    my $self = shift;
+    chdir $self->{_current_dir} unless $self->{_current_dir} eq Cwd::getcwd;
+}
+
+sub _setup_options_pod2usage {
+    pod2usage(1);
+}
+sub _setup_options_version {
+    print "module-setup v$VERSION\n";
+    exit 1;
 }
 
 sub setup_options {
     my($self, %args) = @_;
     $Module::Setup::HAS_TERM = 1;
 
-    pod2usage(2) unless @ARGV;
+    _setup_options_pod2usage unless @ARGV;
 
     my $options = {};
     GetOptions(
-        'init'             => \($options->{init}),
-        'pack'             => \($options->{pack}),
-        'direct'           => \($options->{direct}),
-        'flavor=s'         => \($options->{flavor}),
-        'flavor-class=s'   => \($options->{flavor_class}),
-        'plugin=s@'        => \($options->{plugins}),
-        'target'           => \($options->{target}),
-        'module-setup-dir' => \($options->{module_setup_dir}),
-        version            => sub {
-            print "module-setup v$VERSION\n";
-            exit 1;
-        },
-        help               => sub { pod2usage(1); },
-    ) or pod2usage(2);
+        'init'                         => \($options->{init}),
+        'pack'                         => \($options->{pack}),
+        'direct'                       => \($options->{direct}),
+        'flavor|flavour=s'             => \($options->{flavor}),
+        'flavor-class|flavour-class=s' => \($options->{flavor_class}),
+        'additional=s'                 => \($options->{additional}),
+        'without-additional'           => \($options->{without_additional}),
+        'executable'                   => \($options->{executable}),
+        'plugin=s@'                    => \($options->{plugins}),
+        'target'                       => \($options->{target}),
+        'module-setup-dir'             => \($options->{module_setup_dir}),
+        'devel'                        => \($options->{devel}),
+        'test'                         => \($options->{test}),
+        version                        => \&_setup_options_version,
+        help                           => \&_setup_options_pod2usage,
+    ) or _setup_options_pod2usage;
 
     $self->{options} = $options;
-    $self->{argv}    = \@ARGV;
+    $self->{argv}    = [ @ARGV ];
     $self;
 }
 
@@ -80,6 +105,18 @@ sub _load_argv {
     }
     $self->options->{$name};
 }
+sub _load_argv_module {
+    my $self = shift;
+    $self->_load_argv( module => '' );
+    Carp::croak "module name is required" unless $self->options->{module};
+    $self->options->{module};
+}
+sub _load_argv_flavor {
+    my $self = shift;
+    $self->_load_argv( flavor => sub { $self->select_flavor } );
+    Carp::croak "flavor name is required" unless $self->options->{flavor};
+    $self->options->{flavor};
+}
 
 sub setup_base_dir {
     my $self = shift;
@@ -88,8 +125,9 @@ sub setup_base_dir {
     if ($self->options->{direct}) {
         $path = File::Temp->newdir;
     } else {
-        $path = $self->options->{module_setup_dir} || $ENV{MODULE_SETUP_DIR} || File::HomeDir->my_home;
+        $path = $self->options->{module_setup_dir} || $ENV{MODULE_SETUP_DIR} || Path::Class::Dir->new(File::HomeDir->my_home, '.module-setup');
     }
+    die 'module_setup directory was not able to be discovered.' unless $path;
 
     $self->{base_dir} = Module::Setup::Path->new($path);
     $self->base_dir->init_directories unless $self->base_dir->is_initialized;
@@ -101,35 +139,44 @@ sub run {
     $self->_clear_triggers;
 
     $options->{flavor_class} ||= 'Default';
+    return Module::Setup::Devel->new($self)->run if $options->{devel};
+
     $self->setup_base_dir;
 
-    if ($options->{init}) {
+    if ($options->{init} || (!$options->{pack} && $options->{additional})) {
         $self->_load_argv( flavor => 'default' );
         return $self->create_flavor;
     }
 
-    $self->_load_argv( module => '' );
-    $self->_load_argv( flavor => sub { $self->select_flavor } );
+    $self->_load_argv_module;
+    $self->_load_argv_flavor;
     $self->base_dir->set_flavor($options->{flavor});
 
-    Carp::croak "flavor name is required" unless $options->{flavor};
-    Carp::croak "module name is required" unless $options->{module};
+    if ($options->{additional} && !-d $self->base_dir->flavor->additional->path_to($options->{additional})) {
+        Carp::croak "additional template is no exist: $options->{additional}";
+    }
 
     return $self->pack_flavor if $options->{pack};
 
-    $self->create_flavor unless $self->base_dir->flavor->is_dir;
+    unless ($self->base_dir->flavor->is_dir) {
+        return unless $self->create_flavor;
+    }
 
     $self->load_config;
     $self->load_plugins;
 
     # create skeleton
-    return unless $self->create_skeleton;
+    $self->create_skeleton;
     $self->call_trigger( 'after_create_skeleton' );
 
     # test
     chdir $self->distribute->dist_path;
     $self->call_trigger( 'check_skeleton_directory' );
     $self->call_trigger( 'finalize_create_skeleton' );
+    chdir $self->{_current_dir};
+
+    $self->call_trigger( 'finish_of_run' );
+    $self;
 }
 
 
@@ -187,7 +234,7 @@ sub load_plugins {
             eval "require $pkg"; ## no critic
             Carp::croak $@ if $@;
         }
-        $self->{loaded_plugin}->{$pkg} = $pkg->new( context => $self, config => $self->config );
+        $self->{loaded_plugin}->{$pkg} = $pkg->new( context => $self, config => $config );
     }
 }
 
@@ -202,25 +249,40 @@ sub write_file {
         $path->dir->mkpath;
     }
 
+    my $template;
+    if ($opts->{is_binary}) {
+        $template = pack 'H*', $opts->{template};
+    } else {
+        $template = $opts->{template};
+    }
+
     $self->log("Creating $path");
     my $out = $path->openw;
-    $out->print($opts->{template});
+    $out->print($template);
     $out->close;
 
     chmod oct($opts->{chmod}), $path if $opts->{chmod};
 }
 
 sub install_flavor {
-    my($self, $name, $tmpl) = @_;
+    my($self, $tmpl) = @_;
 
     my $flavor = $self->base_dir->flavor;
+    my $template_path = $flavor->template;
+    if (exists $tmpl->{additional}) {
+        $template_path = Module::Setup::Path::Template->new($flavor->additional->path, $tmpl->{additional});
+        $template_path->path->mkpath;
+    }
+
     my $path;
     if (exists $tmpl->{file} && $tmpl->{file}) {
-        $path = $flavor->template->path_to(split '/', $tmpl->{file});
+        $path = $template_path->path_to(split '/', $tmpl->{file});
     } elsif (exists $tmpl->{dir} && $tmpl->{dir}) {
-        return Module::Setup::Path::Dir->new( $flavor->template->path, split('/', $tmpl->{dir}) )->mkpath;
-    } elsif (exists $tmpl->{plugin} && $tmpl->{plugin}) {
+        return Module::Setup::Path::Dir->new( $template_path->path, split('/', $tmpl->{dir}) )->mkpath;
+    } elsif (exists $tmpl->{plugin} && $tmpl->{plugin} && !exists $tmpl->{additional}) {
         $path = $flavor->plugins->path_to(split '/', $tmpl->{plugin});
+    } else {
+        return;
     }
 
     $self->write_file(+{
@@ -244,20 +306,47 @@ sub create_flavor {
     my $flavor_class = $self->_load_flavor_class($options->{flavor_class});
 
     $self->base_dir->set_flavor($name);
-    Carp::croak "create flavor: $name exists " if $self->base_dir->flavor->is_exists;
+    Carp::croak "create flavor: $name exists " if $self->base_dir->flavor->is_exists && !exists $options->{additional};
+    my $flavor = $flavor_class->new;
+    return unless $flavor->setup_flavor($self);
 
-    my @template = $flavor_class->loader;
+    my @template = $flavor->loader;
     my $config = +{};
+    my $additional_config = +{};
+    if ($options->{additional}) {
+        $additional_config = $self->base_dir->flavor->additional->config->load;
+    }
     for my $tmpl (@template) {
         if (exists $tmpl->{config} && ref($tmpl->{config}) eq 'HASH') {
             $config = $tmpl->{config};
         } else {
-            $self->install_flavor($name, $tmpl);
+            my $additional;
+            if (exists $tmpl->{additional}) {
+                $additional = $tmpl->{additional};
+            } elsif ($options->{additional}) {
+                $additional = $options->{additional};
+            }
+            local $tmpl->{additional} = $additional if $additional; ## no critic;
+            if ($additional) {
+                $additional_config->{$additional} = +{
+                    class => $flavor_class,
+                };
+            }
+            $self->install_flavor($tmpl);
         }
     }
+    $config->{class} = $flavor_class;
+
+    $self->base_dir->flavor->additional->path->mkpath;
+    $self->base_dir->flavor->additional->config->dump($additional_config);
+
+    if ($options->{additional}) {
+        $flavor->setup_additional($self, $config);
+        return 1;
+    }
 
-    # plugins
     $self->base_dir->flavor->plugins->path->mkpath;
+    $self->base_dir->flavor->template->path->mkpath;
 
     if (exists $options->{plugins} && $options->{plugins} && @{ $options->{plugins} }) {
         $config->{plugins} ||= [];
@@ -265,6 +354,8 @@ sub create_flavor {
     }
     $config->{plugins} ||= [];
 
+    $flavor->setup_config($self, $config);
+
     # load plugins
     local $self->{config} = +{
         %{ $config },
@@ -273,7 +364,7 @@ sub create_flavor {
     };
     $self->load_plugins;
 
-    $self->call_trigger( befor_dump_config => $config );
+    $self->call_trigger( before_dump_config => $config );
 
     $self->_clear_triggers;
 
@@ -295,7 +386,9 @@ sub create_skeleton {
         module      => $self->distribute->module,
         dist        => $self->distribute->dist_name,
         module_path => $self->distribute->module_path,
+        module_unix_path => join('/', @{ $self->distribute->package }),
         config      => $config,
+        distribute  => $self->distribute,
         localtime   => scalar localtime,
     };
     $self->call_trigger( after_setup_template_vars => $template_vars);
@@ -314,16 +407,24 @@ sub _collect_flavor_files {
 
     my $base_path = $type->path;
     for my $file ($type->find_files) {
+        my @path = $file->is_dir ? $file->dir_list : ($file->dir->dir_list, $file->basename);
+        while ($path[0] eq '.') { shift @path };
+
         if ($file->is_dir) {
             push @{ $template }, +{
-                dir => join('/', $file->dir_list),
+                dir => join('/', @path),
             };
         } else {
-            my $data = $type->path_to($file)->slurp;
-            push @{ $template }, +{
-                $path_name => join('/', $file->dir->dir_list, $file->basename),
-                template   => $data,
+            my $body = $type->path_to($file)->slurp;
+            my $tmpl = +{
+                $path_name => join('/', @path),
+                template   => $body,
             };
+            if (-B $type->path_to($file)) {
+                $tmpl->{template}  = unpack 'H*', $body;
+                $tmpl->{is_binary} = 1;
+            }
+            push @{ $template }, $tmpl;
         }
     }
 }
@@ -335,15 +436,56 @@ sub pack_flavor {
     my $flavor = $config->{flavor};
 
     my $template = [];
-    $self->_collect_flavor_files($template, file   => $self->base_dir->flavor->template);
-    $self->_collect_flavor_files($template, plugin => $self->base_dir->flavor->plugins);
+    my $flavor_dir = exists $config->{flavor_dir} ? Module::Setup::Path::Flavor->new( $config->{flavor_dir} ) : $self->base_dir->flavor;
+    $self->_collect_flavor_files($template, file   => $flavor_dir->template);
+    $self->_collect_flavor_files($template, plugin => $flavor_dir->plugins);
     push @{ $template }, +{
-        config => YAML::LoadFile($self->base_dir->flavor->config->path),
+        config => YAML::LoadFile($flavor_dir->config->path),
     };
 
+    unless ($config->{without_additional}) {
+        $template = [] if $config->{additional};
+        for my $additional ( $flavor_dir->additional->path->children ) {
+            next unless $additional->is_dir;
+            my $name = $additional->dir_list(-1);
+            next if $config->{additional} && $name ne $config->{additional};
+            my $base_path = Module::Setup::Path::Template->new($flavor_dir->additional->path, $name);
+
+            my $templates = [];
+            $self->_collect_flavor_files($templates, file => $base_path);
+            if ($config->{additional}) {
+                push @{ $template }, @{ $templates };
+            } else {
+                push @{ $template }, map { $_->{additional} = $name; $_ } @{ $templates };
+            }
+        }
+    }
+
+    my $executable_code = '';
+    if ($config->{executable}) {
+        $executable_code = <<EXECUTABLE__;
+#!/bin/env perl
+package main;
+use strict;
+use warnings;
+use Module::Setup;
+
+my \$msetup = Module::Setup->new;
+\$msetup->setup_options;
+\$msetup->options->{direct} = 1;
+\$msetup->options->{flavor_class} = '+$config->{module}';
+{
+    no warnings 'redefine';
+    *Module::Setup::_load_flavor_class = sub { '$config->{module}' };
+    \$msetup->run;
+}
+EXECUTABLE__
+    }
+
     my $eq = '=';
     my $yaml = YAML::Dump(@{ $template });
-    $self->stdout(<<END);
+    $self->stdout(<<FLAVOR__);
+$executable_code
 package $module;
 use strict;
 use warnings;
@@ -360,10 +502,10 @@ ${eq}head1 SYNOPSIS
 
 ${eq}cut
 
-__DATA__
+\__DATA__
 
 $yaml
-END
+FLAVOR__
 }
 
 sub select_flavor {
@@ -413,6 +555,11 @@ sub dialog {
     }
 }
 
+sub system {
+    my($self, @args) = @_;
+    CORE::system(@args);
+}
+
 1;
 __END__
 
@@ -450,11 +597,22 @@ importing redistributed flavor
 
   $ module-setup --init --flavor-class=+MyFlavorCatalystAction new_flavor
 
+install additional template
+
+  $ module-setup --flavor-class=+MyFlavorCatalystDBIC --additional=DBIC catalyst
+
+redistribute pack for additional template
+
+  $ module-setup --pack --additional=DBIC MyFlavorCatalystDBIC catalyst > MyFlavorCatalystDBIC.pm
+
+redistribute pack without additional template
+
+  $ module-setup --pack --without-additional MyFlavorCatalyst catalyst > MyFlavorCatalyst.pm
+
 for git
 
   $ module-setup --plugin=VC::Git Foo::Bar # or edit your ~/.module-setup/flavor/foo/config.yaml
 
-
 =head1 DESCRIPTION
 
 Module::Setup is very simply module start kit.
@@ -491,6 +649,10 @@ Kazuhiro Osawa E<lt>ko@yappo.ne.jpE<gt>
 
 walf443
 
+hidek
+
+tokuhirom
+
 =head1 SEE ALSO
 
 L<Module::Setup::Plugin>, L<module-setup>
