@@ -4,15 +4,15 @@ use warnings;
 use base 'Module::Setup::Flavor';
 
 sub loader {
-    my $class = shift;
-    $class->import_template('Module::Setup::Flavor::Default');
+    my $self = shift;
+    $self->import_template('Module::Setup::Flavor::Default');
 }
 
 1;
 
 =head1
 
-Module::Setup::Flavor::Default - default flavor
+Module::Setup::Flavor::CodeRepos - coderepos flavor
 
 =head1 SYNOPSIS
 
